@@ -1,10 +1,10 @@
 //! Basic building blocks for neural networks.
 //!
 //! Neuronika provides some pre-assembled components, you can either use them individually or
-//! combine them into a bigger architecture. Feel free to take a look at the
-//! [complete list](#layers).
+//! combine them into a bigger architecture. Take a look at the [complete list](#layers) to know
+//! more.
 //!
-//! You can also customise the initialisation of the parameters of such components, and that of any
+//! You can also customize the initialization of the parameters of such components, and that of any
 //! other differentiable variable, by picking the function that best fits your needs from the
 //! [`nn::init`](module@init) module.
 //!
@@ -12,22 +12,22 @@
 //!
 //! # Assembling a neural network
 //!
-//! The suggested way of bulding a model using neuronika's building blocks is to define a struct
+//! The suggested way of building a model using neuronika's building blocks is to define a struct
 //! encapsulating its components.
 //!
-//! The behaviour of the model should be defined by including an appropriate method in its struct
+//! The behavior of the model should be defined by including an appropriate method in its struct
 //! implementation. Such method must specify how the components interact.
 //!
 //! Consider, for the sake of simplicity, a classical *multilayer perceptron* with three dense
 //! layers for a multivariate regression task, let's see what it would look like in neuronika.
 //!
-//! We begin by definining its struct using the provided components.
+//! We begin by defining its struct using the provided components.
 //!
 //! ```
 //! use neuronika::nn;
 //!
-//! // MLP definition.
-//! struct MLP {
+//! // Network definition.
+//! struct NeuralNetwork {
 //!     lin1: nn::Linear,
 //!     lin2: nn::Linear,
 //!     lin3: nn::Linear,     
@@ -38,12 +38,12 @@
 //!
 //! ```
 //! # use neuronika::nn;
-//! # struct MLP {
+//! # struct NeuralNetwork {
 //! #    lin1: nn::Linear,
 //! #    lin2: nn::Linear,
 //! #    lin3: nn::Linear,     
 //! # }
-//! impl MLP {
+//! impl NeuralNetwork {
 //!     // Basic constructor.
 //!     fn new() -> Self {
 //!         Self {
@@ -63,13 +63,13 @@
 //! use neuronika::nn::Learnable;
 //!
 //! # use neuronika::nn;
-//! # struct MLP {
+//! # struct NeuralNetwork {
 //! #     lin1: nn::Linear,
 //! #     lin2: nn::Linear,
 //! #     lin3: nn::Linear,     
 //! # }
-//! impl MLP {
-//!     // MLP behaviour. Notice the presence of the ReLU non-linearity.
+//! impl NeuralNetwork {
+//!     // NeuralNetwork behavior. Notice the presence of the ReLU non-linearity.
 //!     fn forward<I, T, U>(
 //!         &self,
 //!         input: I,
@@ -100,12 +100,12 @@
 //! # use neuronika::nn::Learnable;
 //! # #[cfg(feature = "blas")]
 //! # extern crate blas_src;
-//! # struct MLP {
+//! # struct NeuralNetwork {
 //! #    lin1: nn::Linear,
 //! #    lin2: nn::Linear,
 //! #    lin3: nn::Linear,     
 //! # }
-//! # impl MLP {
+//! # impl NeuralNetwork {
 //! #     // Basic constructor.
 //! #     fn new() -> Self {
 //! #         Self {
@@ -115,8 +115,8 @@
 //! #         }
 //! #     }
 //! # }
-//! # impl MLP {
-//! #     // MLP behaviour. Notice the presence of the ReLU non-linearity.
+//! # impl NeuralNetwork {
+//! #     // NeuralNetwork behavior. Notice the presence of the ReLU non-linearity.
 //! #     fn forward<I, T, U>(
 //! #         &self,
 //! #         input: I,
@@ -136,12 +136,12 @@
 //! #         out3
 //! #     }
 //! # }
-//! let model = MLP::new();
+//! let model = NeuralNetwork::new();
 //!
 //! // Random data to be given in input to the model.
 //! let fictitious_data = neuronika::rand((200, 25));
 //!
-//! let mut out = model.forward(fictitious_data);
+//! let out = model.forward(fictitious_data);
 //! out.forward(); // Always remember to call forward() !
 //! # assert_eq!(out.data().shape(), &[200, 5]);
 //! ```
@@ -157,12 +157,12 @@
 //! # use neuronika::nn::Learnable;
 //! # #[cfg(feature = "blas")]
 //! # extern crate blas_src;
-//! # struct MLP {
+//! # struct NeuralNetwork {
 //! #    lin1: nn::Linear,
 //! #    lin2: nn::Linear,
 //! #    lin3: nn::Linear,     
 //! # }
-//! # impl MLP {
+//! # impl NeuralNetwork {
 //! #     // Basic constructor.
 //! #     fn new() -> Self {
 //! #         Self {
@@ -172,8 +172,8 @@
 //! #         }
 //! #     }
 //! # }
-//! # impl MLP {
-//! #     // MLP behaviour. Notice the presence of the ReLU non-linearity.
+//! # impl NeuralNetwork {
+//! #     // NeuralNetwork behavior. Notice the presence of the ReLU non-linearity.
 //! #     fn forward<I, T, U>(
 //! #         &self,
 //! #         input: I,
@@ -193,20 +193,20 @@
 //! #         out3
 //! #     }
 //! # }
-//! let model = MLP::new();
+//! let model = NeuralNetwork::new();
 //!
 //! let some_other_variable = neuronika::rand((1, 25)).requires_grad();
 //!
-//! // Random perturbated data.
+//! // Random perturbed data.
 //! let fictitious_data = neuronika::rand((200, 25)) + some_other_variable;
 //!
 //! let out = model.forward(fictitious_data);
 //! assert_eq!(out.parameters().len(), 7); // 7 leaf ancestors !
 //! ```
 //!
-//! You can notice how, if we give in input to the multilayer perceptron the result of an
-//! addition operation, in which one of the operands is a differentiable variable, and then
-//! request the *mlp* output's differentiable ancestors, we are given a vector containing 7
+//! You may notice how, if we feed in input to our neural network the result of an addition
+//! operation, in which one of the operands is a differentiable variable, and then
+//! request the network output's differentiable ancestors, we are given a vector containing 7
 //! [`Param`](struct@Param).
 //!
 //! By doing some quick math: 7 = 2 * 3 + 1, and by noticing that each of the three linear layers
@@ -214,27 +214,33 @@
 //! bias vector, we can conclude that the presence of the seventh ancestors is due to the addition
 //! between `fictitious_data` and `some_other_variable`.
 //!
+//! In fact, neuronika automatically tracks all the differentiable leaves that are involved in the
+//! computation of the output variable when assembling the computational graph corresponding to
+//! the issued operations.
+//!
 //! If you need to distinguish between the parameters of a model and another differentiable variable
 //! or between the parameters of several different models, you can use [`ModelStatus`].
 //!
-//! With `ModelStatus` you can build the exact same multilayer perceptron only varying the
-//! implementation so slightly.
+//! With `ModelStatus` you can build the exact same neural network only varying the implementation
+//! so slightly.
 //!
 //! ```
 //!  use neuronika::Param;
 //!  use neuronika::nn::{ModelStatus, Linear};
 //!
-//!  struct MLP {
+//!  struct NeuralNetwork {
 //!     lin1: Linear,
 //!     lin2: Linear,
 //!     lin3: Linear,
-//!     status: ModelStatus,     
+//!     status: ModelStatus,
 //!  }
 //!
-//!  impl MLP {
+//!  impl NeuralNetwork {
 //!      fn new() -> Self {
+//!          // Initialize an empty model status.
 //!          let mut status = ModelStatus::default();
-//!
+//!          
+//!          // We register each component whilst at the same time building the network.
 //!          Self {
 //!              lin1: status.register(Linear::new(25, 30)),
 //!              lin2: status.register(Linear::new(30, 35)),
@@ -242,26 +248,28 @@
 //!              status,
 //!          }
 //!      }
-//!
+//!      
+//!      /// Returns the model's parameters.
 //!      fn parameters(&self) -> Vec<Param> {
+//!          // We are now able to access the parameter of the neural network.
 //!          self.status.parameters()
 //!      }
 //!  }
 //! ```
 //!
-//! Let's verify that the number of registered parameters for the new version of our multilayer
-//! perceptron is indeed 6.
+//! At last, we verify that the number of registered parameters for the new version of our neural
+//! network is indeed 6.
 //!
 //! ```
 //! # use neuronika::Param;
 //! # use neuronika::nn::{ModelStatus, Linear, Learnable};
-//! # struct MLP {
+//! # struct NeuralNetwork {
 //! #     lin1: Linear,
 //! #     lin2: Linear,
 //! #     lin3: Linear,
 //! #     status: ModelStatus,     
 //! # }
-//! # impl MLP {
+//! # impl NeuralNetwork {
 //! #     // Basic constructor.
 //! #     fn new() -> Self {
 //! #         let mut status = ModelStatus::default();
@@ -278,7 +286,7 @@
 //! #        self.status.parameters()
 //! #     }
 //! # }
-//! let model = MLP::new();
+//! let model = NeuralNetwork::new();
 //! assert_eq!(model.parameters().len(), 6);
 //! ```
 //! Do also note that in spite of the introduction of `ModelStatus`, the implementation of the
@@ -286,19 +294,19 @@
 //!
 //! # Train and Eval
 //!
-//! The status of a model determines the behaviour of its components. Certain building blocks, such
-//! as the [`Dropout`], are turned on and off depending on wheter the model is running in *training
+//! The status of a model determines the behavior of its components. Certain building blocks, such
+//! as the [`Dropout`], are turned on and off depending on whether the model is running in *training
 //! mode* or in *inference mode*.
 //!
 //! You can set a network in training mode or in inference mode either by calling [`.train()`] and
 //! [`.eval()`] directly on its output or by using `ModelStatus`.
 //!
-//! The former approach is preferrable, as when multiple models are pipelined, calling `.train()`
+//! The former approach is preferable, as when multiple models are pipelined, calling `.train()`
 //! and `.eval()` directly on the final outputs will switch the statuses of all the models.
 //! Do also note that switching the status by using `ModelStatus` is the only way that allows for
 //! selectively training and evaluating multiple models.
 //!
-//! Let's picture it with simple example.
+//! Let's picture it with a simple example.
 //!
 //! [`.eval()`]: VarDiff::eval()
 //! [`.train()`]: VarDiff::train()
@@ -307,17 +315,23 @@
 //!  use neuronika::Param;
 //!  use neuronika::nn::{ModelStatus, Linear, Dropout};
 //!
-//!  struct MLP {
+//!  struct NeuralNetwork {
 //!     lin1: Linear,
 //!     drop: Dropout,
 //!     lin2: Linear,
 //!     status: ModelStatus,     
 //!  }
 //!
-//!  impl MLP {
+//!  impl NeuralNetwork {
 //!      fn new() -> Self {
 //!          let mut status = ModelStatus::default();
-//!
+//!          
+//!          // Similarly to what we did before, we register the components
+//!          // to the network's status.
+//!          // Now the dropout layer, and every other changeable
+//!          // component, can be directly controlled by interacting
+//!          // with the model itself, as it is synced with the one of
+//!          // ModelStatus.
 //!          Self {
 //!              lin1: status.register(Linear::new(25, 35)),
 //!              drop: status.register(Dropout::new(0.5)),
@@ -329,11 +343,13 @@
 //!      fn parameters(&self) -> Vec<Param> {
 //!          self.status.parameters()
 //!      }
-//!
+//!      
+//!      /// Switches the network in training mode.
 //!      fn train(&self) {
 //!          self.status.train()
 //!      }
-//!
+//!      
+//!      /// Switches the network in inference mode.
 //!      fn eval(&self) {
 //!          self.status.eval()
 //!      }
@@ -380,22 +396,20 @@
 //! the input variable with probability *p* using samples from a Bernoulli distribution.
 use super::{Input, InputBackward, Param};
 use crate::variable::{
-    self,
-    node::{
-        Backward, Data, Dropout as DropoutNode, DropoutBackward as DropoutBackwardNode, Eval,
-        Forward, Gradient, Overwrite,
-    },
-    MatMatMulT, Tensor, Var, VarDiff,
+    self, Backward, Convolve, ConvolveWithGroups, Data, Dropout as DropoutNode,
+    DropoutBackward as DropoutBackwardNode, Eval, Forward, Gradient, MatMatMulT, Overwrite,
+    RawParam, Tensor, Var, VarDiff,
 };
-pub use convolution::{Constant, PaddingMode, Reflective, Replicative, Zero};
-use convolution::{Convolve, ConvolveWithGroups};
+pub use crate::variable::{Constant, PaddingMode, Reflective, Replicative, Zero};
 use ndarray::{Ix1, Ix2, Ix3, Ix4, Ix5};
 use std::{cell::Cell, rc::Rc};
 
-pub(super) mod convolution;
 pub mod init;
 pub mod loss;
 
+#[cfg(feature = "serialize")]
+use serde::{Deserialize, Serialize};
+
 /// A generic parameter of a neural component.
 pub type Learnable<D> = VarDiff<Input<D>, InputBackward<D>>;
 
@@ -408,7 +422,7 @@ pub type Learnable<D> = VarDiff<Input<D>, InputBackward<D>>;
 /// This struct stores all the [`Learnable`] associated to a given model and the model's status. It
 /// is suggested to perform the registration of the layers at the model construction.
 pub struct ModelStatus {
-    params: Vec<Param>,
+    params: Vec<RawParam>,
     train: Rc<Cell<bool>>,
 }
 
@@ -418,14 +432,18 @@ impl ModelStatus {
     ///
     /// Conceptually, this method behaves similarly to [`.parameters()`](VarDiff::parameters()) when
     /// called on the differentiable variable outputted by the network. The key difference lies in
-    /// the fact that, while the differentiable variable's `.parameters()` method would return *all*
+    /// that, while the differentiable variable's `.parameters()` method would return *all*
     /// the differentiable leaves that took part in the computation of the output, possibly also
     /// the weights of another network, `ModelStatus`'s `.parameters()` method returns *only* the
     /// leaves that have been associated with it at the model's instantiation.
     ///
     /// Usually the result of this method is passed to an optimizer.
-    pub fn parameters(&self) -> Vec<Param> {
-        self.params.to_vec()
+    pub fn parameters(&self) -> Vec<Param<'_>> {
+        self.params
+            .iter()
+            .cloned()
+            .map(RawParam::into_param)
+            .collect()
     }
 
     /// Registers a component.
@@ -441,12 +459,12 @@ impl ModelStatus {
 
     /// Sets the status in training mode.
     pub fn train(&self) {
-        <Self as Eval>::train(&self)
+        <Self as Eval>::train(self)
     }
 
     /// Sets the status in inference mode.
     pub fn eval(&self) {
-        <Self as Eval>::eval(&self)
+        <Self as Eval>::eval(self)
     }
 }
 
@@ -507,7 +525,7 @@ where
 /// Registration for neuronika's components.
 pub trait Register {
     /// Registers `self`'s parameters to the model's  status parameters `params`.
-    fn register_params(&self, params: &mut Vec<Param>);
+    fn register_params(&self, params: &mut Vec<RawParam>);
 
     /// Register `self`'s status to the model's status state `status`.
     fn register_status(&mut self, status: Rc<Cell<bool>>);
@@ -564,7 +582,7 @@ impl Register for Dropout {
         self.status = status;
     }
 
-    fn register_params(&self, _: &mut Vec<Param>) {}
+    fn register_params(&self, _: &mut Vec<RawParam>) {}
 }
 
 /// Applies a **linear transformation** to the incoming data.
@@ -572,6 +590,7 @@ impl Register for Dropout {
 /// ```text
 /// ʏ = xAᵀ + b
 /// ```
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct Linear {
     pub weight: Learnable<Ix2>,
     pub bias: Learnable<Ix1>,
@@ -589,14 +608,14 @@ impl Linear {
     /// The learnable weight of the layer is of shape `(out_features, in_features)`. The learnable
     /// bias of the layer is of shape `out_features`.
     ///
-    /// The values for both the weight and bias are initialised from *U(-k, k)* where
+    /// The values for both the weight and bias are initialized from *U(-k, k)* where
     /// `k = (1. / in_features as f32).sqrt()`.
     pub fn new(in_features: usize, out_features: usize) -> Self {
-        let mut weight = Input::new(Tensor::zeros((out_features, in_features))).requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_features)).requires_grad();
+        let weight = Input::new(Tensor::zeros((out_features, in_features))).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_features)).requires_grad();
         let k = (1. / (in_features as f32)).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self { weight, bias }
     }
@@ -623,16 +642,17 @@ impl Linear {
 
 impl Register for Linear {
     /// Registers the weight and the bias of this `Linear` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
 }
 
 /// A **long short-term memory (LSTM)** cell.
-#[allow(clippy::clippy::upper_case_acronyms)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[allow(clippy::upper_case_acronyms)]
 pub struct LSTMCell {
     pub weight_ih: Learnable<Ix2>,
     pub weight_hh: Learnable<Ix2>,
@@ -649,7 +669,7 @@ impl LSTMCell {
     ///
     /// * `hidden_size` - number of features in the hidden state.
     ///
-    /// All the weight and biases are initialised from *U(-k, k)* where
+    /// All the weight and biases are initialized from *U(-k, k)* where
     /// `k = (1. / hidden_size as f32).sqrt()`.
     pub fn new(input_size: usize, hidden_size: usize) -> Self {
         let (weight_ih_shape, weight_hh_shape, bias_shape) = {
@@ -660,16 +680,16 @@ impl LSTMCell {
                 xhidden_size,
             )
         };
-        let mut weight_ih = Input::new(Tensor::zeros(weight_ih_shape)).requires_grad();
-        let mut weight_hh = Input::new(Tensor::zeros(weight_hh_shape)).requires_grad();
-        let mut bias_ih = Input::new(Tensor::zeros(bias_shape)).requires_grad();
-        let mut bias_hh = Input::new(Tensor::zeros(bias_shape)).requires_grad();
+        let weight_ih = Input::new(Tensor::zeros(weight_ih_shape)).requires_grad();
+        let weight_hh = Input::new(Tensor::zeros(weight_hh_shape)).requires_grad();
+        let bias_ih = Input::new(Tensor::zeros(bias_shape)).requires_grad();
+        let bias_hh = Input::new(Tensor::zeros(bias_shape)).requires_grad();
 
         let k = 1. / (hidden_size as f32).sqrt();
-        init::uniform(&mut weight_ih, -k, k);
-        init::uniform(&mut weight_hh, -k, k);
-        init::uniform(&mut bias_ih, -k, k);
-        init::uniform(&mut bias_hh, -k, k);
+        init::uniform(&weight_ih, -k, k);
+        init::uniform(&weight_hh, -k, k);
+        init::uniform(&bias_ih, -k, k);
+        init::uniform(&bias_hh, -k, k);
 
         Self {
             weight_ih,
@@ -735,18 +755,19 @@ impl LSTMCell {
 
 impl Register for LSTMCell {
     /// Registers the weights and the biases of this LSTMCell instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight_hh.parameters());
-        params.extend(self.weight_ih.parameters());
-        params.extend(self.bias_hh.parameters());
-        params.extend(self.bias_ih.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight_hh.register_params(params);
+        self.weight_ih.register_params(params);
+        self.bias_hh.register_params(params);
+        self.bias_ih.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
 }
 
 /// A **gated recurrent unit (GRU)** cell.
-#[allow(clippy::clippy::upper_case_acronyms)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[allow(clippy::upper_case_acronyms)]
 pub struct GRUCell {
     pub weight_ih: Learnable<Ix2>,
     pub weight_hh: Learnable<Ix2>,
@@ -763,7 +784,7 @@ impl GRUCell {
     ///
     /// * `hidden_size` - number of features in the hidden state.
     ///
-    /// All the weight and biases are initialised from *U(-k, k)* where
+    /// All the weight and biases are initialized from *U(-k, k)* where
     /// `k = (1. / hidden_size as f32).sqrt()`.
     pub fn new(input_size: usize, hidden_size: usize) -> Self {
         let (weight_ih_shape, weight_hh_shape, bias_shape) = {
@@ -774,16 +795,16 @@ impl GRUCell {
                 xhidden_size,
             )
         };
-        let mut weight_ih = Input::new(Tensor::zeros(weight_ih_shape)).requires_grad();
-        let mut weight_hh = Input::new(Tensor::zeros(weight_hh_shape)).requires_grad();
-        let mut bias_ih = Input::new(Tensor::zeros(bias_shape)).requires_grad();
-        let mut bias_hh = Input::new(Tensor::zeros(bias_shape)).requires_grad();
+        let weight_ih = Input::new(Tensor::zeros(weight_ih_shape)).requires_grad();
+        let weight_hh = Input::new(Tensor::zeros(weight_hh_shape)).requires_grad();
+        let bias_ih = Input::new(Tensor::zeros(bias_shape)).requires_grad();
+        let bias_hh = Input::new(Tensor::zeros(bias_shape)).requires_grad();
 
         let k = 1. / (hidden_size as f32).sqrt();
-        init::uniform(&mut weight_ih, -k, k);
-        init::uniform(&mut weight_hh, -k, k);
-        init::uniform(&mut bias_ih, -k, k);
-        init::uniform(&mut bias_hh, -k, k);
+        init::uniform(&weight_ih, -k, k);
+        init::uniform(&weight_hh, -k, k);
+        init::uniform(&bias_ih, -k, k);
+        init::uniform(&bias_hh, -k, k);
 
         Self {
             weight_ih,
@@ -838,11 +859,11 @@ impl GRUCell {
 
 impl Register for GRUCell {
     /// Registers the weights and the biases of this `GRUCell` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight_hh.parameters());
-        params.extend(self.weight_ih.parameters());
-        params.extend(self.bias_hh.parameters());
-        params.extend(self.bias_ih.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight_hh.register_params(params);
+        self.weight_ih.register_params(params);
+        self.bias_hh.register_params(params);
+        self.bias_ih.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
@@ -851,6 +872,7 @@ impl Register for GRUCell {
 /// Applies a **temporal convolution** over an input signal composed of several input planes.
 ///
 /// See also [`GroupedConv1d`].
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct Conv1d<Pad: PaddingMode> {
     pub padding: usize,
     pub padding_mode: Pad,
@@ -881,7 +903,7 @@ impl<Pad: PaddingMode> Conv1d<Pad> {
     /// * `dilation` - controls the spacing between the kernel points, a number for this
     /// one-dimensional case.
     ///
-    /// The weight and the bias of the layer are initialised from *U(-k, k)* where
+    /// The weight and the bias of the layer are initialized from *U(-k, k)* where
     /// `k = (1. /(in_channels * kernel_size) as f32).sqrt()`.
     pub fn new(
         in_channels: usize,
@@ -892,13 +914,13 @@ impl<Pad: PaddingMode> Conv1d<Pad> {
         stride: usize,
         dilation: usize,
     ) -> Self {
-        let mut weight =
+        let weight =
             Input::new(Tensor::zeros((out_channels, in_channels, kernel_size))).requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (1. / (in_channels * kernel_size) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -952,9 +974,9 @@ impl<Pad: PaddingMode> Conv1d<Pad> {
 
 impl<Pad: PaddingMode> Register for Conv1d<Pad> {
     /// Registers the weight and the bias of this `Conv1d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
@@ -962,6 +984,7 @@ impl<Pad: PaddingMode> Register for Conv1d<Pad> {
 
 /// Applies a **grouped temporal convolution** over an input signal composed of several input
 /// planes.
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct GroupedConv1d<Pad: PaddingMode> {
     pub padding: usize,
     pub padding_mode: Pad,
@@ -1003,9 +1026,9 @@ impl<Pad: PaddingMode> GroupedConv1d<Pad> {
     /// both subsequently concatenated.
     ///* at `groups = in_channels`, each input channel is convolved with its own set of filters.
     ///
-    /// The weight and the bias of the layer are initialised from *U(-k, k)* where
+    /// The weight and the bias of the layer are initialized from *U(-k, k)* where
     /// `k = (groups /(in_channels * kernel_size) as f32).sqrt()`.
-    #[allow(clippy::clippy::too_many_arguments)]
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         in_channels: usize,
         out_channels: usize,
@@ -1016,17 +1039,17 @@ impl<Pad: PaddingMode> GroupedConv1d<Pad> {
         dilation: usize,
         groups: usize,
     ) -> Self {
-        let mut weight = Input::new(Tensor::zeros((
+        let weight = Input::new(Tensor::zeros((
             out_channels,
             in_channels / groups,
             kernel_size,
         )))
         .requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (groups as f32 / (in_channels * kernel_size) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -1082,9 +1105,9 @@ impl<Pad: PaddingMode> GroupedConv1d<Pad> {
 
 impl<Pad: PaddingMode> Register for GroupedConv1d<Pad> {
     /// Registers the weight and the bias of this `GroupedConv1d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
@@ -1093,6 +1116,7 @@ impl<Pad: PaddingMode> Register for GroupedConv1d<Pad> {
 /// Applies a **spatial convolution** over an input signal composed of several input planes.
 ///
 /// See also [`GroupedConv2d`].
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct Conv2d<Pad: PaddingMode> {
     pub padding: (usize, usize),
     pub padding_mode: Pad,
@@ -1123,7 +1147,7 @@ impl<Pad: PaddingMode> Conv2d<Pad> {
     /// * `dilation` - controls the spacing between the kernel points, a 2-tuple for this
     /// two-dimensional case.
     ///
-    /// The weight and the bias are initialised from *U(-k, k)* where
+    /// The weight and the bias are initialized from *U(-k, k)* where
     /// `k = (1. /(in_channels * kernel_w * kernel_h) as f32).sqrt()`.
     pub fn new(
         in_channels: usize,
@@ -1135,18 +1159,18 @@ impl<Pad: PaddingMode> Conv2d<Pad> {
         dilation: (usize, usize),
     ) -> Self {
         let (kernel_h, kernel_w) = kernel_size;
-        let mut weight = Input::new(Tensor::zeros((
+        let weight = Input::new(Tensor::zeros((
             out_channels,
             in_channels,
             kernel_h,
             kernel_w,
         )))
         .requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (1. / (in_channels * kernel_h * kernel_w) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -1206,15 +1230,16 @@ impl<Pad: PaddingMode> Conv2d<Pad> {
 
 impl<Pad: PaddingMode> Register for Conv2d<Pad> {
     /// Registers the weight and the bias of this `Conv2d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
 }
 
 /// Applies a **spatial grouped convolution** over an input signal composed of several input planes.
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct GroupedConv2d<Pad: PaddingMode> {
     pub padding: (usize, usize),
     pub padding_mode: Pad,
@@ -1256,9 +1281,9 @@ impl<Pad: PaddingMode> GroupedConv2d<Pad> {
     /// and both subsequently concatenated.
     /// * at `groups = in_channels`, each input channel is convolved with its own set of filters.
     ///
-    /// The weight and the bias of the layer are initialised from *U(-k, k)* where
+    /// The weight and the bias of the layer are initialized from *U(-k, k)* where
     /// `k = (groups /(in_channels * kernel_h * kernel_w) as f32).sqrt()`.
-    #[allow(clippy::clippy::too_many_arguments)]
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         in_channels: usize,
         out_channels: usize,
@@ -1270,18 +1295,18 @@ impl<Pad: PaddingMode> GroupedConv2d<Pad> {
         groups: usize,
     ) -> Self {
         let (kernel_h, kernel_w) = kernel_size;
-        let mut weight = Input::new(Tensor::zeros((
+        let weight = Input::new(Tensor::zeros((
             out_channels,
             in_channels,
             kernel_h,
             kernel_w,
         )))
         .requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (groups as f32 / (in_channels * kernel_h * kernel_w) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -1343,9 +1368,9 @@ impl<Pad: PaddingMode> GroupedConv2d<Pad> {
 
 impl<Pad: PaddingMode> Register for GroupedConv2d<Pad> {
     /// Registers the weight and the bias of this `GroupedConv2d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
@@ -1354,6 +1379,7 @@ impl<Pad: PaddingMode> Register for GroupedConv2d<Pad> {
 /// Applies a **volumetric convolution** over an input signal composed of several input planes.
 ///
 /// See also [`GroupedConv3d`].
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct Conv3d<Pad: PaddingMode> {
     pub padding: (usize, usize, usize),
     pub padding_mode: Pad,
@@ -1384,7 +1410,7 @@ impl<Pad: PaddingMode> Conv3d<Pad> {
     /// * `dilation` - controls the spacing between the kernel points, a 3-tuple for this
     /// three-dimensional case.
     ///
-    /// The weight and the bias of the layer are initialised from *U(-k, k)* where
+    /// The weight and the bias of the layer are initialized from *U(-k, k)* where
     /// `k = (1. /(in_channels * kernel_d * kernel_w * kernel_h) as f32).sqrt()`.
     pub fn new(
         in_channels: usize,
@@ -1396,7 +1422,7 @@ impl<Pad: PaddingMode> Conv3d<Pad> {
         dilation: (usize, usize, usize),
     ) -> Self {
         let (kernel_d, kernel_h, kernel_w) = kernel_size;
-        let mut weight = Input::new(Tensor::zeros((
+        let weight = Input::new(Tensor::zeros((
             out_channels,
             in_channels,
             kernel_d,
@@ -1404,11 +1430,11 @@ impl<Pad: PaddingMode> Conv3d<Pad> {
             kernel_w,
         )))
         .requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (1. / (in_channels * kernel_d * kernel_h * kernel_w) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -1470,9 +1496,9 @@ impl<Pad: PaddingMode> Conv3d<Pad> {
 
 impl<Pad: PaddingMode> Register for Conv3d<Pad> {
     /// Registers the weight and the bias of this `Conv3d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
@@ -1480,6 +1506,7 @@ impl<Pad: PaddingMode> Register for Conv3d<Pad> {
 
 /// Applies a **grouped volumetric convolution** over an input signal composed of several input
 /// planes.
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 pub struct GroupedConv3d<Pad: PaddingMode> {
     pub padding: (usize, usize, usize),
     pub padding_mode: Pad,
@@ -1521,7 +1548,7 @@ impl<Pad: PaddingMode> GroupedConv3d<Pad> {
     /// and both subsequently concatenated.
     /// * at `groups = in_channels`, each input channel is convolved with its own set of filters.
     ///
-    /// The weight and the bias are initialised from *U(-k, k)* where
+    /// The weight and the bias are initialized from *U(-k, k)* where
     /// `k = (groups /(in_channels * kernel_d * kernel_h * kernel_w) as f32).sqrt()`.
     #[allow(clippy::too_many_arguments)]
     pub fn new(
@@ -1535,7 +1562,7 @@ impl<Pad: PaddingMode> GroupedConv3d<Pad> {
         groups: usize,
     ) -> Self {
         let (kernel_d, kernel_h, kernel_w) = kernel_size;
-        let mut weight = Input::new(Tensor::zeros((
+        let weight = Input::new(Tensor::zeros((
             out_channels,
             in_channels,
             kernel_d,
@@ -1543,11 +1570,11 @@ impl<Pad: PaddingMode> GroupedConv3d<Pad> {
             kernel_w,
         )))
         .requires_grad();
-        let mut bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
+        let bias = Input::new(Tensor::zeros(out_channels)).requires_grad();
 
         let k = (1. / (in_channels * kernel_d * kernel_h * kernel_w) as f32).sqrt();
-        init::uniform(&mut weight, -k, k);
-        init::uniform(&mut bias, -k, k);
+        init::uniform(&weight, -k, k);
+        init::uniform(&bias, -k, k);
 
         Self {
             padding,
@@ -1609,9 +1636,9 @@ impl<Pad: PaddingMode> GroupedConv3d<Pad> {
 
 impl<Pad: PaddingMode> Register for GroupedConv3d<Pad> {
     /// Registers the weight and the bias of this `GroupedConv3d` instance.
-    fn register_params(&self, params: &mut Vec<Param>) {
-        params.extend(self.weight.parameters());
-        params.extend(self.bias.parameters());
+    fn register_params(&self, params: &mut Vec<RawParam>) {
+        self.weight.register_params(params);
+        self.bias.register_params(params);
     }
 
     fn register_status(&mut self, _: Rc<Cell<bool>>) {}
